"""Provider-axis clients, generated from the platform OpenAPI specs."""

from .digitalhuman import AsyncDigitalhuman as AsyncDigitalhuman
from .digitalhuman import Digitalhuman as Digitalhuman
from .dreamina import AsyncDreamina as AsyncDreamina
from .dreamina import Dreamina as Dreamina
from .fish import AsyncFish as AsyncFish
from .fish import Fish as Fish
from .flux import AsyncFlux as AsyncFlux
from .flux import Flux as Flux
from .hailuo import AsyncHailuo as AsyncHailuo
from .hailuo import Hailuo as Hailuo
from .happyhorse import AsyncHappyhorse as AsyncHappyhorse
from .happyhorse import Happyhorse as Happyhorse
from .localization import AsyncLocalization as AsyncLocalization
from .localization import Localization as Localization
from .luma import AsyncLuma as AsyncLuma
from .luma import Luma as Luma
from .maestro import AsyncMaestro as AsyncMaestro
from .maestro import Maestro as Maestro
from .minimax import AsyncMinimax as AsyncMinimax
from .minimax import Minimax as Minimax
from .nano_banana import AsyncNanoBanana as AsyncNanoBanana
from .nano_banana import NanoBanana as NanoBanana
from .producer import AsyncProducer as AsyncProducer
from .producer import Producer as Producer
from .seedance import AsyncSeedance as AsyncSeedance
from .seedance import Seedance as Seedance
from .seedream import AsyncSeedream as AsyncSeedream
from .seedream import Seedream as Seedream
from .suno import AsyncSuno as AsyncSuno
from .suno import Suno as Suno
from .wan import AsyncWan as AsyncWan
from .wan import Wan as Wan

__all__ = [
    "Digitalhuman",
    "AsyncDigitalhuman",
    "Dreamina",
    "AsyncDreamina",
    "Fish",
    "AsyncFish",
    "Flux",
    "AsyncFlux",
    "Hailuo",
    "AsyncHailuo",
    "Happyhorse",
    "AsyncHappyhorse",
    "Localization",
    "AsyncLocalization",
    "Luma",
    "AsyncLuma",
    "Maestro",
    "AsyncMaestro",
    "Minimax",
    "AsyncMinimax",
    "NanoBanana",
    "AsyncNanoBanana",
    "Producer",
    "AsyncProducer",
    "Seedance",
    "AsyncSeedance",
    "Seedream",
    "AsyncSeedream",
    "Suno",
    "AsyncSuno",
    "Wan",
    "AsyncWan",
]
